#ifndef MATRIX_H_
#define MATRIX_H_

#include <sstream>
#include <iostream>
#include <string>
#include <vector>
#include <list>
#include <algorithm>

#include "MeanComputer.h"
#include "Addable.h"
#include "Subtractable.h"
#include "Number.h"
#include "Printable.h"

using namespace std;

/**
 * Rappresenta una matrice di elementi a priori generici.
 */
template<class T> class Matrix: public Addable, public Subtractable, public Printable{
    public:
        /** Crea una matrice identità 1x1. */
        Matrix();
        /** Crea una matrice identità rowsxcolumns. */
        Matrix(int rows, int columns);
        /** Costruttore di copia. */
        Matrix(Matrix<T>* toCopy);
        /** Distruttore di default. */
        virtual ~Matrix();
        /** Ritorna una stringa che rappresenta la matrice. */
        virtual string toString();
        /** Modifica un elemento. */
        void set(int row, int column, T value);
        /** Ritorna un elemento. */
        T get(int row, int column);
        /** Esegue la somma elemento per elemento. */
        Matrix<T>* add(Addable* number);
        /** Moltiplica per una costante. */
        Matrix<T>* multiply(int k);
        /** Esegue la differenza elemento per elemento. */
        Matrix<T>* subtract(Subtractable* number);
        /** Ritorna una lista con gli elementi della matrice. */
        list<T>* toList();
        /** Ritorna la media degli elementi della matrice. */
        double getMean();
        /** Ritorna il massimo fra gli elementi della matrice. */
        T getMax();
        /** Applica una funzione a tutti gli elementi della matrice. */
        template<class F> Matrix<T>* apply(F f);
        
        // Ridefinizione degli operatori
        T& operator() (int row, int column) const;
        T& operator() (int row, int column);
        Matrix<T>* operator* (int k);
        
    private:
        /** Numero di righe della matrice. */
        int rows;
        /** Numero di colonne della matrice. */
        int columns;
        /** Contenitore degli elementi. */
        vector< vector<T>* >* elements;
};

// l'implementazione della classe è qui per una limitazione del compilatore.
#include "Matrix.cpp"

#endif /*MATRIX_H_*/
